@@ -10,6 +10,45 @@ TODO:
 import numpy as np
 
 class Broker():
+    """
+    Virtual broker to simulate trading environment.
+
+
+    Attributes
+    ----------
+    broker_config : dict
+        the broker configuration dictionary
+    
+    leverage : int
+        the leverage on the account (default 1)
+    
+    commission : int
+        the brokers commission as a percentage (default 0)
+        
+    spread : int
+        the bid/ask spread (default 0)
+    
+    utils : class
+        the broker utilities class
+    
+    home_curreny : str
+        the home currency of the account (used for retrieving quote data)
+
+
+    Methods
+    -------
+    place_order(order_details):
+        Places an order with the virtual broker.
+    
+    open_position(order_no, candle, limit_price = None):
+        Opens an order to enter the market as a trade. 
+    
+    update_positions(candle):
+        Updates orders and open positions based on current candle. 
+
+    
+    
+    """
     
     def __init__(self, broker_config, utils):
         self.leverage           = 1
@@ -88,56 +127,60 @@ class Broker():
             self.cancelled_orders[order_no] = self.pending_positions[order_no]
     
     
-    def update_positions(self, candle):
+    def update_positions(self, candle, instrument):
         ''' 
             Updates orders and open positions based on current candle. 
         '''
+
         # Tally for positions opened this update
         opened_positions = 0
         
         # Update pending positions
         closing_orders = []
         for order_no in self.pending_positions:
-            if self.pending_positions[order_no]['order_time'] != candle.name:
-                if self.pending_positions[order_no]['order_type'] == 'market':
-                    # Market order type
-                    self.open_position(order_no, candle)
-                    opened_positions += 1
+            # Filter orders by instrument type since candle is instrument specific            
+            if self.pending_positions[order_no]['instrument'] == instrument:
+                if self.pending_positions[order_no]['order_time'] != candle.name:
+                    if self.pending_positions[order_no]['order_type'] == 'market':
+                        # Market order type
+                        self.open_position(order_no, candle)
+                        opened_positions += 1
+                    
+                    elif self.pending_positions[order_no]['order_type'] == 'stop-limit':
+                        # Stop-limit order type
+                        # Check if order_stop_price has been reached yet
+                        
+                        if candle.Low < self.pending_positions[order_no]['order_stop_price'] < candle.High:
+                            # order_stop_price has been reached, change order type to 'limit'
+                            self.pending_positions[order_no]['order_type'] = 'limit'
+                        
+                    # This is in a separate if statement, as stop-limit order may
+                    # eventually be changed to limit orders
+                    if self.pending_positions[order_no]['order_type'] == 'limit':
+                        # Limit order type
+                        if self.pending_positions[order_no]['size'] > 0:
+                            if candle.Low < self.pending_positions[order_no]['order_limit_price']:
+                                self.open_position(order_no, candle, 
+                                                   self.pending_positions[order_no]['order_limit_price'])
+                                opened_positions += 1
+                        else:
+                            if candle.High > self.pending_positions[order_no]['order_limit_price']:
+                                self.open_position(order_no, candle, 
+                                                   self.pending_positions[order_no]['order_limit_price'])
+                                opened_positions += 1
+                                
+                                
+                if self.pending_positions[order_no]['order_type'] == 'close':
+                    related_order = self.pending_positions[order_no]['related_orders']
+                    self.close_position(self.pending_positions[order_no]['instrument'],
+                                        candle, 
+                                        candle.Close,
+                                        order_no = related_order
+                                        )
+                    opened_positions += 1 # To remove from pending orders
+                    closing_orders.append(order_no)
+        
                 
-                elif self.pending_positions[order_no]['order_type'] == 'stop-limit':
-                    # Stop-limit order type
-                    # Check if order_stop_price has been reached yet
-                    
-                    if candle.Low < self.pending_positions[order_no]['order_stop_price'] < candle.High:
-                        # order_stop_price has been reached, change order type to 'limit'
-                        self.pending_positions[order_no]['order_type'] = 'limit'
-                    
-                # This is in a separate if statement, as stop-limit order may
-                # eventually be changed to limit orders
-                if self.pending_positions[order_no]['order_type'] == 'limit':
-                    # Limit order type
-                    if self.pending_positions[order_no]['size'] > 0:
-                        if candle.Low < self.pending_positions[order_no]['order_limit_price']:
-                            self.open_position(order_no, candle, 
-                                               self.pending_positions[order_no]['order_limit_price'])
-                            opened_positions += 1
-                    else:
-                        if candle.High > self.pending_positions[order_no]['order_limit_price']:
-                            self.open_position(order_no, candle, 
-                                               self.pending_positions[order_no]['order_limit_price'])
-                            opened_positions += 1
-                            
-                            
-            if self.pending_positions[order_no]['order_type'] == 'close':
-                related_order = self.pending_positions[order_no]['related_orders']
-                self.close_position(self.pending_positions[order_no]['instrument'],
-                                    candle, 
-                                    candle.Close,
-                                    order_no = related_order
-                                    )
-                opened_positions += 1 # To remove from pending orders
-                closing_orders.append(order_no)
-                    
         # Remove position from pending positions
         if opened_positions > 0:
             # For orders that were opened
@@ -152,83 +195,86 @@ class Broker():
             for order_no in closing_orders:
                 self.pending_positions.pop(order_no, 0)
         
+        
         # Update trailing stops
         # For other methods, move the stop update to an external function
         # Can this be moved into the loop below?
         for order_no in self.open_positions:
-            if self.open_positions[order_no]['stop_type'] == 'trailing_stop':
-                # Trailing stop loss is enabled, check if price has moved SL
-                
-                if self.open_positions[order_no]['stop_distance'] is not None:
-                    pip_value = self.utils.get_pip_ratio(self.open_positions[order_no]['instrument'])
-                    pip_distance = self.open_positions[order_no]['stop_distance']
-                    distance = pip_distance*pip_value
+            if self.open_positions[order_no]['instrument'] == instrument:
+                if self.open_positions[order_no]['stop_type'] == 'trailing_stop':
+                    # Trailing stop loss is enabled, check if price has moved SL
                     
-                else:
-                    distance = abs(self.open_positions[order_no]['entry_price'] - \
-                                   self.open_positions[order_no]['stop_loss'])
-                
-
-                if self.open_positions[order_no]['size'] > 0:
-                    # long position, stop loss only moves up
-                    new_stop = candle.High - distance
-                    if new_stop > self.open_positions[order_no]['stop_loss']:
-                        self.open_positions[order_no]['stop_loss'] = new_stop
+                    if self.open_positions[order_no]['stop_distance'] is not None:
+                        pip_value = self.utils.get_pip_ratio(self.open_positions[order_no]['instrument'])
+                        pip_distance = self.open_positions[order_no]['stop_distance']
+                        distance = pip_distance*pip_value
+                        
+                    else:
+                        distance = abs(self.open_positions[order_no]['entry_price'] - \
+                                       self.open_positions[order_no]['stop_loss'])
                     
-                else:
-                    # short position, stop loss only moves down
-                    new_stop = candle.Low + distance
-                    if new_stop < self.open_positions[order_no]['stop_loss']:
-                        self.open_positions[order_no]['stop_loss'] = new_stop
+    
+                    if self.open_positions[order_no]['size'] > 0:
+                        # long position, stop loss only moves up
+                        new_stop = candle.High - distance
+                        if new_stop > self.open_positions[order_no]['stop_loss']:
+                            self.open_positions[order_no]['stop_loss'] = new_stop
+                        
+                    else:
+                        # short position, stop loss only moves down
+                        new_stop = candle.Low + distance
+                        if new_stop < self.open_positions[order_no]['stop_loss']:
+                            self.open_positions[order_no]['stop_loss'] = new_stop
         
         # Update self.open_positions
         open_position_orders = list(self.open_positions.keys())
         unrealised_PL  = 0        # Un-leveraged value
         for order_no in open_position_orders:
-            if self.open_positions[order_no]['size'] > 0:
-                if self.open_positions[order_no]['stop_loss'] is not None and \
-                    candle.Low < self.open_positions[order_no]['stop_loss']:
-                    # Stop loss hit
-                    self.close_position(self.open_positions[order_no]['instrument'], 
-                                        candle, 
-                                        self.open_positions[order_no]['stop_loss'],
-                                        order_no)
-                elif self.open_positions[order_no]['take_profit'] is not None and \
-                    candle.High > self.open_positions[order_no]['take_profit']:
-                    # Take Profit hit
-                    self.close_position(self.open_positions[order_no]['instrument'], 
-                                        candle, 
-                                        self.open_positions[order_no]['take_profit'],
-                                        order_no)
+            if self.open_positions[order_no]['instrument'] == instrument:
+                if self.open_positions[order_no]['size'] > 0:
+                    if self.open_positions[order_no]['stop_loss'] is not None and \
+                        candle.Low < self.open_positions[order_no]['stop_loss']:
+                        # Stop loss hit
+                        self.close_position(self.open_positions[order_no]['instrument'], 
+                                            candle, 
+                                            self.open_positions[order_no]['stop_loss'],
+                                            order_no)
+                    elif self.open_positions[order_no]['take_profit'] is not None and \
+                        candle.High > self.open_positions[order_no]['take_profit']:
+                        # Take Profit hit
+                        self.close_position(self.open_positions[order_no]['instrument'], 
+                                            candle, 
+                                            self.open_positions[order_no]['take_profit'],
+                                            order_no)
+                    else:
+                        # Position is still open, update value of holding
+                        size        = self.open_positions[order_no]['size']
+                        entry_price = self.open_positions[order_no]['entry_price']
+                        price       = candle.Close
+                        HCF         = self.open_positions[order_no]['HCF']
+                        position_value = size*(price - entry_price)*HCF
+                        unrealised_PL += position_value
+                
                 else:
-                    # Position is still open, update value of holding
-                    size        = self.open_positions[order_no]['size']
-                    entry_price = self.open_positions[order_no]['entry_price']
-                    price       = candle.Close
-                    HCF         = self.open_positions[order_no]['HCF']
-                    position_value = size*(price - entry_price)*HCF
-                    unrealised_PL += position_value
-            
-            else:
-                if self.open_positions[order_no]['stop_loss'] is not None and \
-                    candle.High > self.open_positions[order_no]['stop_loss']:
-                    self.close_position(self.open_positions[order_no]['instrument'], 
-                                        candle, 
-                                        self.open_positions[order_no]['stop_loss'],
-                                        order_no)
-                elif self.open_positions[order_no]['take_profit'] is not None and \
-                    candle.Low < self.open_positions[order_no]['take_profit']:
-                    self.close_position(self.open_positions[order_no]['instrument'], 
-                                        candle, 
-                                        self.open_positions[order_no]['take_profit'],
-                                        order_no)
-                else:
-                    size        = self.open_positions[order_no]['size']
-                    entry_price = self.open_positions[order_no]['entry_price']
-                    price       = candle.Close
-                    HCF         = self.open_positions[order_no]['HCF']
-                    position_value = size*(price - entry_price)*HCF
-                    unrealised_PL += position_value
+                    if self.open_positions[order_no]['stop_loss'] is not None and \
+                        candle.High > self.open_positions[order_no]['stop_loss']:
+                        self.close_position(self.open_positions[order_no]['instrument'], 
+                                            candle, 
+                                            self.open_positions[order_no]['stop_loss'],
+                                            order_no)
+                    elif self.open_positions[order_no]['take_profit'] is not None and \
+                        candle.Low < self.open_positions[order_no]['take_profit']:
+                        self.close_position(self.open_positions[order_no]['instrument'], 
+                                            candle, 
+                                            self.open_positions[order_no]['take_profit'],
+                                            order_no)
+                    else:
+                        size        = self.open_positions[order_no]['size']
+                        entry_price = self.open_positions[order_no]['entry_price']
+                        price       = candle.Close
+                        HCF         = self.open_positions[order_no]['HCF']
+                        position_value = size*(price - entry_price)*HCF
+                        unrealised_PL += position_value
         
         # Update margin available
         self.update_margin(candle.Close)
@@ -301,7 +347,31 @@ class Broker():
     
     def get_open_positions(self, instrument=None):
         ''' Returns the open positions in the account. '''
-        return self.open_positions
+        
+        open_positions = {}
+        
+        if instrument is not None:
+            for order_no in self.open_positions:
+                if self.open_positions[order_no]['instrument'] == instrument:
+                    open_positions[order_no] = self.open_positions[order_no]
+        else:
+            open_positions = self.open_positions.copy()
+        
+        return open_positions
+    
+    def get_cancelled_orders(self, instrument = None):
+        ''' Returns cancelled orders. '''
+        
+        cancelled_orders = {}
+        
+        if instrument is not None:
+            for order_no in self.cancelled_orders:
+                if self.cancelled_orders[order_no]['instrument'] == instrument:
+                    cancelled_orders[order_no] = self.cancelled_orders[order_no]
+        else:
+            cancelled_orders = self.cancelled_orders.copy()
+        
+        return cancelled_orders
     
     
     def add_funds(self, amount):
